unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, RegExpr, synedit, synedittypes,  ExtCtrls,  EditPlugAPI,
  EditPlugAPITypes, EditPlugFindUtils;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    bFind: TButton;
    Panel3: TPanel;
    edSearch: TMemo;
    Panel4: TPanel;
    lblSearch: TLabel;
    lblFolder: TLabel;
    edFolder: TEdit;
    edFileType: TEdit;
    lblFileType: TLabel;
    cbAllOpenFiles: TCheckBox;
    lblWarning: TLabel;
    cbRegex: TCheckBox;
    cbCaseSensitive: TCheckBox;
    procedure bFindClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure cbAllOpenFilesClick(Sender: TObject);
  private
    { Private declarations }
    FirstShow: boolean;
    procedure FindInAllOpenFiles;
    procedure FindInOpenFilesRegex(openfilecnt: integer);
    procedure FindInOpenFiles(openfilecnt: integer);
    procedure FindInFolder;
    procedure FindInFolderRegular;
    procedure FindInFolderRegex;
    function SearchFile(substr: string; syn: TSynedit;
      filename: string): boolean;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  gPlugId: HINST;

implementation

{$R *.DFM}


{----- EXPORTS ----------------------------------------------------------------}

procedure plgSetId(id: HINST); cdecl;
begin
  gPlugId:= id;
end; exports plgSetId;

function plgShow: HWND; cdecl;
begin
  if not assigned(form1) then begin
    form1:= tform1.create(nil);
    epSetParentToMainWin(form1.handle);
    form1.show;
    epMoveWinToTopRight(form1);
  end else begin
    form1.show;
  end;
  result:= form1.Handle;
end; exports plgShow;

procedure plgWinFree; cdecl;
begin
  if assigned(form1) then begin
    form1.Free;
    form1:=nil;
  end;
end; exports plgWinFree;

function plgGetWinHandle: HWND; cdecl;
begin
  result:=0;
  if assigned(form1) then result:= form1.Handle;
end; exports plgGetWinHandle;

{------------------------------------------------------------------------------}

procedure NoMatch;
begin
  showmessage('Could not find match');
  epActiveEdRemoveSel;
end;

function CreateFileMaskList(s: string): TStringList;
begin
  result := TStringList.create;
  result.text := StringReplace(s, ';', #13#10, [rfReplaceAll]);
end;

// returns true if match found
function TForm1.SearchFile(substr: string; syn: TSynedit;
  filename: string): boolean;
var
  startpos: integer;
  txt, line: string;
  bufcoord: TBufferCoord;
  coord: TBufCoord;
  position: dword;
  casesensitive: boolean;
begin
  result := false;
  casesensitive := cbCaseSensitive.checked;
  syn.Lines.LoadFromFile(filename);
  if casesensitive then
    txt := syn.text
  else begin
    txt := lowercase(syn.text);
    substr := lowercase(substr);
  end;
  position := NextPos(substr, txt);
  if position > 0 then begin
    result:= true;
    repeat
      startpos := position-1;
      syn.SelStart := startpos;
      bufcoord:= syn.CharIndexToRowCol(startpos);
      coord.char := bufcoord.char;
      coord.line := bufcoord.line -1;
      line := syn.Lines[bufcoord.line-1];
      epAddSearchResult(pchar(filename), pchar(line), coord, length(substr));
      epAppProcessMessages;
      position := NextPos(substr, txt, position+1);
    until position = 0;
  end else begin

  end;
end;

// returns true if match found
function SearchFileRegex(r: TRegExpr; syn: TSynedit; filename: string): boolean;
var
  startpos: integer;
  s: string;
  bufcoord: TBufferCoord;
  coord: TBufCoord;
begin
  result := false;
  syn.Lines.LoadFromFile(filename);
  r.InputString := syn.text;
  if r.Exec then begin
    result:= true;
    repeat
      startpos := r.matchpos[0]-1;
      syn.SelStart := startpos;
      bufcoord:= syn.CharIndexToRowCol(startpos);
      coord.char := bufcoord.char;
      coord.line := bufcoord.line -1;
      s := syn.Lines[bufcoord.line-1];
      epAddSearchResult(pchar(filename), pchar(s), coord, length(r.match[0]));
      epAppProcessMessages;
    until not r.ExecNext;
  end else begin

  end;
end;

function EnsureTrailSlash(folder: string): string;
begin
  result := '';
  if folder = '' then exit;
  if folder[length(folder)] <> '\' then result := folder + '\';
end;

function CreateFileListFromMasks(folder: string; masks: TStringList): TStringList;
var
  i: integer;
  searchResult : TSearchRec;
begin
  result := TStringList.create;
  for i := 0 to masks.count - 1 do begin
    if FindFirst(folder + masks.Strings[i],
                 faAnyFile, searchResult) = 0 then
    begin
      repeat
        if searchResult.Attr <> faDirectory then
            result.add(searchResult.Name);
      until FindNext(searchResult) <> 0;
      // Must free up resources used by these successful finds
      FindClose(searchResult);
    end;
  end;
end;

procedure ShowDidNotFindResults;
begin
  showmessage('did not find any results');
end;

procedure TForm1.FindInFolderRegular;
var
  sl, files: TStringList;
  i: integer;
  syn: TSynedit;
  found: boolean;
  foundcount: integer;
  folder: string;
begin
  foundcount := 0;
  sl := CreateFileMaskList(edFileType.text);
  folder := EnsureTrailSlash(edFolder.text);
  files := CreateFileListFromMasks(folder, sl);
  syn := TSynEdit.create(self);
  try
    syn.BeginUpdate;
    syn.Parent := self;
    syn.Visible := false;
    for i := 0 to files.count -1 do begin
       found := SearchFile(edSearch.text, syn, folder + files.Strings[i]);
       if found then inc(foundcount);
    end;
    if foundcount = 0 then ShowDidNotFindResults;
  finally
    syn.EndUpdate;
    syn.free;
    files.free;
    sl.free;
  end;

end;

procedure TForm1.FindInFolderRegex;
var
  sl, files: TStringList;
  i: integer;
  r: TRegExpr;
  syn: TSynedit;
  found: boolean;
  foundcount: integer;
  folder: string;
begin
  foundcount := 0;
  sl := CreateFileMaskList(edFileType.text);
  folder := EnsureTrailSlash(edFolder.text);
  files := CreateFileListFromMasks(folder, sl);
  r := TRegExpr.Create;
  syn := TSynEdit.create(self);
  try
    syn.BeginUpdate;
    syn.Parent := self;
    syn.Visible := false;
    r.Expression := ProcessEscapeChars(edSearch.text);
    // case sensitivity
    r.ModifierI := not cbCaseSensitive.checked;
    for i := 0 to files.count -1 do begin
       found := SearchFileRegex(r, syn, folder + files.Strings[i]);
       if found then inc(foundcount);
    end;
    if foundcount = 0 then ShowDidNotFindResults;
  finally
    r.free;
    syn.EndUpdate;
    syn.free;
    files.free;
    sl.free;
  end;

end;

procedure TForm1.FindInFolder;
begin
  if edFolder.text = '' then begin
    showmessage('Enter a folder to search through');
    exit;
  end;
  if edFileType.text = '' then begin
    showmessage('Enter a file type (masks) to search, separated by semi colons');
    exit;
  end;

  if cbRegex.checked then FindInFolderRegex else FindInFolderRegular;
end;

function CreateOpenFileList(count: integer): TStringList;
var
  i: integer;
  fname: string;
begin
  result := TStringList.Create;
  if count < 1 then exit;
  for i := 0 to count - 1 do begin
    fname := epGetEdFileNameAsString(i);
    if fname <> '' then
      if FileExists(fname) then result.add(fname);
  end;
end;

procedure TForm1.FindInOpenFilesRegex(openfilecnt: integer);
var
  files: TStringList;
  i: integer;
  r: TRegExpr;
  syn: TSynedit;
  found: boolean;
  foundcount: integer;
begin
  foundcount := 0;
  files := CreateOpenFileList(openfilecnt);
  r := TRegExpr.Create;
  syn := TSynEdit.create(self);
  try
    syn.BeginUpdate;
    syn.Parent := self;
    syn.Visible := false;
    r.Expression := ProcessEscapeChars(edSearch.text);
    r.ModifierI := not cbCaseSensitive.checked;
    for i := 0 to files.count -1 do begin
       found := SearchFileRegex(r, syn, files.Strings[i]);
       if found then inc(foundcount);
    end;
    if foundcount = 0 then ShowDidNotFindResults;
  finally
    r.free;
    syn.EndUpdate;
    syn.free;
    files.free;
  end;

end;

procedure TForm1.FindInOpenFiles(openfilecnt: integer);
var
  files: TStringList;
  i: integer;
  syn: TSynedit;
  found: boolean;
  foundcount: integer;
begin
  foundcount := 0;
  files := CreateOpenFileList(openfilecnt);
  syn := TSynEdit.create(self);
  try
    syn.BeginUpdate;
    syn.Parent := self;
    syn.Visible := false;
    for i := 0 to files.count -1 do begin
       found := SearchFile(edSearch.text, syn, files.Strings[i]);
       if found then inc(foundcount);
    end;
    if foundcount = 0 then ShowDidNotFindResults;
  finally
    syn.EndUpdate;
    syn.free;
    files.free;
  end;
end;

procedure TForm1.FindInAllOpenFiles;
var filecnt: integer;
begin
  filecnt := epGetEdCount;
  if filecnt < 1 then begin
    showmessage('No files are open');
    exit;
  end;

  if epFilesModified then begin
    lblWarning.Caption := 'Warning: save files first for correct results';
  end;

  if cbRegex.checked then
    FindInOpenFilesRegex(filecnt) else FindInOpenFiles(filecnt);
end;

procedure TForm1.bFindClick(Sender: TObject);
var oldcaption1, oldcaption2: string;
begin
  lblWarning.Caption := '';
  oldcaption1 := bFind.caption;
  oldcaption2 := self.caption;
  self.caption := 'Busy searching...';
  bFind.caption := 'Busy..';
  bFind.Enabled := false;
  if edSearch.text = '' then begin
    showmessage('Enter a search phrase');
    exit;
  end;
  epClearSearchResults;
  //epSearchResultsBeginUpdate;
  if cbAllOpenFiles.Checked then FindInAllOpenFiles else FindInFolder;
  //epSearchResultsEndUpdate;
  bFind.caption := oldcaption1;
  self.caption := oldcaption2;
  bFind.Enabled := true;
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  FirstShow:= true;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if FirstShow then begin
    FirstShow := false;
  end;
end;

procedure TForm1.cbAllOpenFilesClick(Sender: TObject);
var cb: TCheckBox;
begin
  if sender is TCheckBox then begin
    cb := sender as tcheckbox;
    edFileType.enabled := not cb.checked;
    lblFileType.enabled := not cb.checked;
    lblFolder.enabled := not cb.checked;
    edFolder.enabled := not cb.checked;
  end;
end;

end.
